"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rpc_1 = require("./rpc");
var MuRemoteRPCServer = (function () {
    function MuRemoteRPCServer(rpc) {
        this.rpc = rpc;
    }
    return MuRemoteRPCServer;
}());
exports.MuRemoteRPCServer = MuRemoteRPCServer;
var uniqueNumber = (function () {
    var next = 0;
    return function () { return next++; };
})();
var MuRPCClient = (function () {
    function MuRPCClient(client, schema) {
        this._callbacks = {};
        this.sessionId = client.sessionId;
        this.client = client;
        this.schema = schema;
        var schemaUnfolded = rpc_1.unfoldRPCProtocolSchema(schema);
        this._callProtocol = client.protocol(schemaUnfolded[0]);
        this._responseProtocol = client.protocol(schemaUnfolded[1]);
        this._errorProtocol = client.protocol(rpc_1.MuRPCErrorProtocolSchema);
        this.server = new MuRemoteRPCServer(this._createPRCToServer());
    }
    MuRPCClient.prototype._createPRCToServer = function () {
        var _this = this;
        var rpc = {};
        Object.keys(this.schema.server).forEach(function (method) {
            rpc[method] = function (arg, next) {
                var id = uniqueNumber();
                _this._callbacks[id] = next;
                _this._callProtocol.server.message[method]({
                    base: _this.schema.server[method][0].clone(arg),
                    id: id,
                });
            };
        });
        return rpc;
    };
    MuRPCClient.prototype.configure = function (spec) {
        var _this = this;
        this._callProtocol.configure({
            message: (function () {
                var handlers = {};
                Object.keys(_this.schema.client).forEach(function (method) {
                    handlers[method] = function (_a) {
                        var base = _a.base, id = _a.id;
                        spec.rpc[method](base, function (err, response) {
                            if (err) {
                                _this._errorProtocol.server.message.error({ message: err, id: id });
                            }
                            else {
                                _this._responseProtocol.server.message[method]({
                                    base: _this.schema.client[method][1].clone(response),
                                    id: id,
                                });
                            }
                        });
                    };
                });
                return handlers;
            })(),
        });
        this._responseProtocol.configure({
            message: (function () {
                var handlers = {};
                Object.keys(_this.schema.server).forEach(function (method) {
                    handlers[method] = function (_a) {
                        var base = _a.base, id = _a.id;
                        _this._callbacks[id](undefined, _this.schema.server[method][1].clone(base));
                        delete _this._callbacks[id];
                    };
                });
                return handlers;
            })(),
            ready: function () {
                if (spec && spec.ready) {
                    spec.ready();
                }
            },
            close: function () {
                if (spec && spec.close) {
                    spec.close();
                }
            },
        });
        this._errorProtocol.configure({
            message: {
                error: function (_a) {
                    var message = _a.message, id = _a.id;
                    delete _this._callbacks[id];
                },
            },
        });
    };
    return MuRPCClient;
}());
exports.MuRPCClient = MuRPCClient;
//# sourceMappingURL=client.js.map